@@ -7,7 +7,7 @@ use test::Bencher;
 use rand::seq::SliceRandom;
 
 use keeshond::gameloop::{GameInfo, GameControl, GameControlOptions};
-use keeshond::scene::{BaseScene, SimpleScene, NullSceneType, Entity, Component, ThinkerSystem, ThinkerArgs};
+use keeshond::scene::{BaseScene, SimpleScene, NullSceneType, Entity, Component, ThinkerSystem, ThinkerArgs, EntityComponent};
 
 const NUM_ITER : usize = 10_000;
 
@@ -165,15 +165,27 @@ impl ThinkerSystem<NullSceneType> for PosVelThinker
 fn make_scene_multi_component() -> SimpleScene
 {
     let mut my_scene = SimpleScene::new();
-    
+    let mut positions = Vec::with_capacity(NUM_ITER);
+    let mut velocities = Vec::with_capacity(NUM_ITER);
+
     for _ in 0..NUM_ITER
     {
-        let new_entity = my_scene.add_entity();
-        
-        my_scene.add_component(&new_entity, Position::new());
-        my_scene.add_component(&new_entity, Velocity::new());
+        let entity = my_scene.add_entity();
+        positions.push(EntityComponent
+        {
+            entity : entity.clone(),
+            component : Position::new()
+        });
+        velocities.push(EntityComponent
+        {
+            entity,
+            component : Velocity::new()
+        });
     }
     
+    my_scene.add_multiple_components(positions);
+    my_scene.add_multiple_components(velocities);
+    
     my_scene.add_thinker_system(Box::new(PosVelThinker {}));
     
     my_scene
@@ -200,14 +212,19 @@ impl ThinkerSystem<NullSceneType> for PosVelSingleThinker
 fn make_scene_single_component() -> SimpleScene
 {
     let mut my_scene = SimpleScene::new();
-    
+    let mut positions = Vec::with_capacity(NUM_ITER);
+
     for _ in 0..NUM_ITER
     {
-        let new_entity = my_scene.add_entity();
-        
-        my_scene.add_component(&new_entity, PositionVelocity::new());
+        let entity = my_scene.add_entity();
+        positions.push(EntityComponent
+        {
+            entity : entity.clone(),
+            component : PositionVelocity::new()
+        });
     }
-    
+
+    my_scene.add_multiple_components(positions);
     my_scene.add_thinker_system(Box::new(PosVelSingleThinker {}));
     
     my_scene
@@ -235,36 +252,31 @@ impl ThinkerSystem<NullSceneType> for PosVelOrderedThinker
     }
 }
 
-fn make_scene_removal() -> SimpleScene
-{
-    let mut my_scene = SimpleScene::new();
-    
-    my_scene.add_thinker_system(Box::new(PosVelThinker {}));
-    
-    for _ in 0..NUM_ITER
-    {
-        let new_entity = my_scene.add_entity();
-        
-        my_scene.add_component(&new_entity, Position::new());
-        my_scene.add_component(&new_entity, Velocity::new());
-    }
-    
-    my_scene
-}
-
 fn make_scene_removal_ordered() -> SimpleScene
 {
     let mut my_scene = SimpleScene::new();
-    
-    my_scene.add_thinker_system(Box::new(PosVelOrderedThinker {}));
-    
+    let mut positions = Vec::with_capacity(NUM_ITER);
+    let mut velocities = Vec::with_capacity(NUM_ITER);
+
     for _ in 0..NUM_ITER
     {
-        let new_entity = my_scene.add_entity();
-        
-        my_scene.add_component(&new_entity, PositionOrdered::new());
-        my_scene.add_component(&new_entity, VelocityOrdered::new());
+        let entity = my_scene.add_entity();
+        positions.push(EntityComponent
+        {
+            entity : entity.clone(),
+            component : PositionOrdered::new()
+        });
+        velocities.push(EntityComponent
+        {
+            entity,
+            component : VelocityOrdered::new()
+        });
     }
+
+    my_scene.add_multiple_components(positions);
+    my_scene.add_multiple_components(velocities);
+
+    my_scene.add_thinker_system(Box::new(PosVelOrderedThinker {}));
     
     my_scene
 }
@@ -371,13 +383,26 @@ fn bench_build_rebuild_2_cycles(bencher : &mut Bencher)
         
         for gen in 1..3
         {
+            let mut positions = Vec::with_capacity(NUM_ITER);
+            let mut velocities = Vec::with_capacity(NUM_ITER);
+
             for _ in 0..NUM_ITER
             {
-                let new_entity = my_scene.add_entity();
-                
-                my_scene.add_component(&new_entity, Position::new());
-                my_scene.add_component(&new_entity, Velocity::new());
+                let entity = my_scene.add_entity();
+                positions.push(EntityComponent
+                {
+                    entity : entity.clone(),
+                    component : Position::new()
+                });
+                velocities.push(EntityComponent
+                {
+                    entity,
+                    component : Velocity::new()
+                });
             }
+
+            my_scene.add_multiple_components(positions);
+            my_scene.add_multiple_components(velocities);
             
             for i in 0..NUM_ITER
             {
@@ -393,7 +418,7 @@ fn bench_build_delete_all(bencher : &mut Bencher)
     bencher.iter(||
     {
         let mut dummy = GameControl::new(&GameInfo::default(), GameControlOptions::headless()).unwrap();
-        let mut my_scene = make_scene_removal();
+        let mut my_scene = make_scene_multi_component();
         
         for i in 0..NUM_ITER
         {
@@ -429,7 +454,7 @@ fn bench_build_delete_gradual(bencher : &mut Bencher)
     bencher.iter(||
     {
         let mut dummy = GameControl::new(&GameInfo::default(), GameControlOptions::headless()).unwrap();
-        let mut my_scene = make_scene_removal();
+        let mut my_scene = make_scene_multi_component();
         let rng = &mut rand::thread_rng();
         
         let mut ids : Vec<usize> = (0..NUM_ITER).collect();
