@@ -159,7 +159,7 @@ impl AlSoundPreparer
 
 impl DataPreparer<Sound, AlSoundHandle> for AlSoundPreparer
 {
-    fn prepare(&mut self, data : &mut Sound, _id : DataId) -> AlSoundHandle
+    fn prepare(&mut self, data : &mut Sound, _id : DataId<Sound>) -> AlSoundHandle
     {
         let context_ref : &Context = &self.context.borrow();
         let buffer_raw;
@@ -189,7 +189,7 @@ impl DataPreparer<Sound, AlSoundHandle> for AlSoundPreparer
             buffer
         }
     }
-    fn unprepare(&mut self, _prepared : &mut AlSoundHandle, _id : DataId)
+    fn unprepare(&mut self, _prepared : &mut AlSoundHandle, _id : DataId<Sound>)
     {
         
     }
@@ -202,8 +202,8 @@ pub trait Audio
     fn set_unit_scale(&mut self, scale : f32);
     fn listener_position(&self) -> (f32, f32);
     fn set_listener_position(&mut self, x : f32, y : f32);
-    fn play_sound(&mut self, sound_id : DataId) -> Option<Voice>;
-    fn play_sound_with(&mut self, sound_id : DataId, voice_info : &VoiceInfo) -> Option<Voice>;
+    fn play_sound(&mut self, sound_id : DataId<Sound>) -> Option<Voice>;
+    fn play_sound_with(&mut self, sound_id : DataId<Sound>, voice_info : &VoiceInfo) -> Option<Voice>;
     fn stop_voice(&mut self, voice : &Voice) -> bool;
     fn voice(&self, voice : &Voice) -> Option<VoiceInfo>;
     fn set_voice(&mut self, voice : &Voice, voice_info : &VoiceInfo) -> bool;
@@ -276,12 +276,12 @@ impl Audio for NullAudio
     #[allow(unused_variables)]
     fn set_listener_position(&mut self, x : f32, y : f32) {}
     #[allow(unused_variables)]
-    fn play_sound(&mut self, sound_id : DataId) -> Option<Voice>
+    fn play_sound(&mut self, sound_id : DataId<Sound>) -> Option<Voice>
     {
         Some(Voice { slot : 0, generation : 0 })
     }
     #[allow(unused_variables)]
-    fn play_sound_with(&mut self, sound_id : DataId, voice_info : &VoiceInfo) -> Option<Voice>
+    fn play_sound_with(&mut self, sound_id : DataId<Sound>, voice_info : &VoiceInfo) -> Option<Voice>
     {
         Some(Voice { slot : 0, generation : 0 })
     }
@@ -314,7 +314,7 @@ pub struct AlAudio
     listener_position : (f32, f32),
     sources : Vec<StaticSource>,
     voice_generations : Vec<u64>,
-    voice_sounds : Vec<DataId>,
+    voice_sounds : Vec<DataId<Sound>>,
     voice_pending : BitSet<u32>,
     voice_info : Vec<VoiceInfo>,
     al_sound_store : PreparedStore<Sound, AlSoundHandle>
@@ -354,7 +354,7 @@ impl AlAudio
             
             sources.push(source);
             voice_generations.push(0);
-            voice_sounds.push(0);
+            voice_sounds.push(DataId::new(0));
             voice_info.push(VoiceInfo::default());
         }
         
@@ -577,12 +577,12 @@ impl Audio for AlAudio
         }
     }
     
-    fn play_sound(&mut self, sound_id : DataId) -> Option<Voice>
+    fn play_sound(&mut self, sound_id : DataId<Sound>) -> Option<Voice>
     {
         self.play_sound_with(sound_id, &VoiceInfo::default())
     }
     
-    fn play_sound_with(&mut self, sound_id : DataId, voice_info : &VoiceInfo) -> Option<Voice>
+    fn play_sound_with(&mut self, sound_id : DataId<Sound>, voice_info : &VoiceInfo) -> Option<Voice>
     {
         let voice_result = self.find_voice();
         
