@@ -1,9 +1,9 @@
 //! Gameloop and [gameloop::GameControl] context, which is used to control the game globally
 
 use crate::input::Input;
-use crate::scene::{Scene, BaseScene, SceneType, SceneConfig, NullSpawnable};
-use crate::audio::{Audio, AlAudio, NullAudio, AudioError};
-use crate::renderer::{Renderer, RendererError, RendererNewInfo, ViewportMode};
+use crate::scene::{BaseScene, NullSpawnable, Scene, SceneConfig, SceneType};
+use crate::audio::{AlAudio, Audio, AudioError, NullAudio};
+use crate::renderer::{Renderer, RendererError, RendererNewInfo, ViewportMode, WindowScaleMode};
 #[cfg(feature = "imgui_base")] use crate::imgui_handler::ImGuiHandler;
 #[cfg(feature = "default_logger")] use crate::doglog;
 
@@ -15,12 +15,12 @@ use sdl2::messagebox;
 #[cfg(feature = "graphical_panic")] use std::panic::PanicInfo;
 #[cfg(feature = "graphical_panic")] use backtrace::Backtrace;
 
-use keeshond_datapack::{DataMultistore, DataObject, DataId};
+use keeshond_datapack::{DataId, DataMultistore, DataObject};
 use keeshond_datapack::source::SourceManager;
 
 #[cfg(feature = "imgui_base")] use imgui;
 
-use std::time::{Instant, Duration};
+use std::time::{Duration, Instant};
 use std::rc::Rc;
 use std::cell::RefCell;
 use std::collections::HashMap;
@@ -30,6 +30,12 @@ use std::sync::{Arc, Mutex, MutexGuard};
 use crate::doglog::ConsoleHistory;
 
 const MILLISECONDS_PER_NANOSECOND : f64 = 0.000001;
+const BUSY_WAIT_MARGIN : u64 = 2; // For smooth results on Windows. Linux can probably get away with 1.
+const AUTOMATIC_LERP_CAP_MULTIPLIER : f64 = 1.05;
+const MAX_POST_SLOWDOWN_CATCHUP : u32 = 10;
+const MAX_LOGIC_FRAME_OVERRUN : u32 = 8;
+const MAX_FRAME_JIFFY_SUBTRACT : u32 = 1000;
+const MAX_FPS_STAT_CATCHUP : u64 = 3;
 const LOW_ENERGY_TIMER_MIN : u32 = 60;
 const LOW_ENERGY_DRAW_RATE : u32 = 2;
 
@@ -145,14 +151,12 @@ fn read_options() -> getopts::Matches
     options.optflag("w", "window", "Run in windowed mode");
     options.optopt("b", "backends", &format!("Select the renderer backends to use, separated by spaces and ordered by preference. Default list: \"{}\"", backends), "BACKEND_LIST");
     options.optflag("", "interpolate", "When an alternate refresh rate is detected, unlock display framerate from logic and enable frame interpolation.");
-    options.optflag("", "no-interpolate", "Force disable frame interpolation and lock display rate to logic rate. Use with a fixed refresh rate equal to game's refresh rate.");
-    options.optflag("", "force-interpolate", "Force enable frame interpolation. Not recommended to use with a fixed refresh rate equal to game's logic rate!");
-    options.optopt("", "interp-cap", "Upper framerate limit for frame interpolation, determined by FPS_LIMIT (-1 for default, 0 to disable).", "FPS_LIMIT");
+    options.optflag("", "no-interpolate", "Force disable frame interpolation and lock display rate to logic rate. Use when monitor's refresh rate equals game's logic rate.");
+    options.optflag("", "force-interpolate", "Force enable frame interpolation. Not recommended if monitor's refresh rate equals game's logic rate!");
+    options.optopt("", "interp-cap", "Upper framerate limit for frame interpolation, determined by FPS_LIMIT (\"auto\" for default, \"off\" to disable).", "FPS_LIMIT");
     options.optflag("", "kiosk", "Enable kiosk mode, which forces fullscreen and disables closing the window. Note that the user may still be able to close the program depending on your operating environment's global shortcuts.");
     
-    let match_result = options.parse(&args[1..]);
-    
-    match match_result
+    match options.parse(&args[1..])
     {
         Ok(matches) =>
         {
@@ -203,17 +207,22 @@ fn apply_config(gameinfo : &mut GameInfo)
         gameinfo.frame_interpolation = FrameInterpolationMode::Force;
     }
 
-    match matches.opt_get::<f64>("interp-cap")
+    match matches.opt_str("interp-cap")
     {
-        Ok(Some(cap)) =>
+        Some(cap) =>
         {
-            if cap < 1.0 && cap >= 0.0
+            match cap.as_str()
             {
-                gameinfo.frame_interpolation_cap = None;
-            }
-            else if cap >= 1.0
-            {
-                gameinfo.frame_interpolation_cap = Some(cap);
+                "auto" => { gameinfo.frame_interpolation_cap = FrameInterpolationCap::Automatic; }
+                "off" => { gameinfo.frame_interpolation_cap = FrameInterpolationCap::None; }
+                _ =>
+                {
+                    match cap.parse::<f64>()
+                    {
+                        Ok(value) => { gameinfo.frame_interpolation_cap = FrameInterpolationCap::Manual(value); }
+                        Err(_) => { gameinfo.frame_interpolation_cap = FrameInterpolationCap::Automatic; }
+                    }
+                }
             }
         },
         _ => ()
@@ -228,17 +237,30 @@ fn apply_config(gameinfo : &mut GameInfo)
 }
 
 /// Whether to use frame interpolation, which results in cleaner motion on high refresh rate displays for games that support it
-#[derive(Copy, Clone, Debug)]
+#[derive(Copy, Clone, Debug, PartialEq, Eq)]
 pub enum FrameInterpolationMode
 {
     /// Never do frame interpolation (recommended for games that don't support frame interpolation)
     Off,
     /// Do frame interpolation if the display refresh rate doesn't match the game's target framerate (recommended for games that support frame interpolation)
     Automatic,
-    /// Always do frame interpolation (not recommended)
+    /// Always do frame interpolation (not recommended, but since automatic detection doesn't work for web builds, you might want to include this as an in-game option)
     Force
 }
 
+/// The framerate cap to use for when frame interpolation is enabled
+#[derive(Copy, Clone, Debug, PartialEq)]
+pub enum FrameInterpolationCap
+{
+    /// Sets the framerate limit to slightly above the monitor's current refresh rate. This improves the smoothness of
+    ///  lerping when there is significant logic processing.
+    Automatic,
+    /// Sets the framerate limit to the specified amount (between 5-1000 FPS)
+    Manual(f64),
+    /// Uncaps the framerate entirely (not recommended)
+    None
+}
+
 /// Properties and metadata for this game
 #[derive(Clone)]
 pub struct GameInfo
@@ -251,9 +273,8 @@ pub struct GameInfo
     pub base_width : u32,
     /// Base resolution height of the game graphics
     pub base_height : u32,
-    /// The default magnification level, defaulting to 1x. Mostly useful with
-    ///  the Pixel viewport mode.
-    pub default_zoom : u32,
+    /// The window scale strategy to use when creating the window
+    pub default_window_scale : WindowScaleMode,
     /// Whether to use texture filtering. Set to false for pixel art style games.
     pub texture_filtering : bool,
     /// The viewport mode to use, which determines how the game scales to different
@@ -268,7 +289,7 @@ pub struct GameInfo
     pub frame_interpolation : FrameInterpolationMode,
     /// Optional framerate cap for frame interpolation mode, useful for keeping the
     ///  display framerate below the maximum supported VRR rate.
-    pub frame_interpolation_cap : Option<f64>,
+    pub frame_interpolation_cap : FrameInterpolationCap,
     /// Whether to display the mouse cursor when it is over the window.
     pub cursor_visible : bool,
     /// Whether unused resources should be automatically unloaded between scenes.
@@ -305,7 +326,7 @@ impl Default for GameInfo
             friendly_name : "Untitled",
             base_width : 1280,
             base_height : 720,
-            default_zoom : 1,
+            default_window_scale : WindowScaleMode::MultiplierDpiScaled(1.0),
             texture_filtering : true,
             viewport_mode : ViewportMode::Independent,
             fullscreen : false,
@@ -316,7 +337,7 @@ impl Default for GameInfo
             low_energy_app : false,
             target_framerate : 60.0,
             frame_interpolation : FrameInterpolationMode::Off,
-            frame_interpolation_cap : None,
+            frame_interpolation_cap : FrameInterpolationCap::Automatic,
             backends : crate::renderer::available_backends(),
             log_stats : true,
             log_packages : Vec::new()
@@ -466,7 +487,7 @@ pub struct GameControl
 {
     target_framerate : f64,
     frame_interpolation : FrameInterpolationMode,
-    interpolation_cap : Option<f64>,
+    interpolation_cap : FrameInterpolationCap,
     low_energy_app : bool,
     low_energy_timer : u32,
     auto_unload : bool,
@@ -496,8 +517,6 @@ impl GameControl
     {   
         let source_manager = Rc::new(RefCell::new(SourceManager::new()));
         
-        let default_zoom = gameinfo.default_zoom.max(1).min(8);
-        
         let mut resources = DataMultistore::new(source_manager.clone());
         
         let mut renderer_new_info = None;
@@ -532,7 +551,7 @@ impl GameControl
             {
                 width : gameinfo.base_width,
                 height : gameinfo.base_height,
-                default_zoom,
+                default_window_scale : gameinfo.default_window_scale,
                 texture_filtering : gameinfo.texture_filtering,
                 kiosk_mode : gameinfo.kiosk_mode,
                 fullscreen : gameinfo.fullscreen,
@@ -597,12 +616,7 @@ impl GameControl
     
     pub fn set_target_framerate(&mut self, new_framerate : f64)
     {
-        if new_framerate < 1.0 || new_framerate >= 1000.0
-        {
-            panic!("Target framerate must be between 1 and 1000.");
-        }
-        
-        self.target_framerate = new_framerate;
+        self.target_framerate = new_framerate.clamp(5.0, 1000.0);
     }
     
     pub fn frame_interpolation(&self) -> FrameInterpolationMode
@@ -615,22 +629,19 @@ impl GameControl
         self.frame_interpolation = mode;
     }
     
-    pub fn frame_interpolation_cap(&self) -> Option<f64>
+    pub fn frame_interpolation_cap(&self) -> FrameInterpolationCap
     {
         self.interpolation_cap
     }
     
-    pub fn set_frame_interpolation_cap(&mut self, framerate : Option<f64>)
+    pub fn set_frame_interpolation_cap(&mut self, mut cap : FrameInterpolationCap)
     {
-        if let Some(cap) = framerate
+        if let FrameInterpolationCap::Manual(value) = cap
         {
-            if cap < 1.0 || cap >= 1000.0
-            {
-                panic!("Target framerate cap must be between 1 and 1000.");
-            }
+            cap = FrameInterpolationCap::Manual(value.clamp(5.0, 1000.0));
         }
         
-        self.interpolation_cap = framerate;
+        self.interpolation_cap = cap;
     }
 
     pub fn frame_interpolation_active(&self) -> bool
@@ -647,7 +658,7 @@ impl GameControl
                     return false;
                 }
 
-                self.target_framerate.round() as i32 != current_refresh_rate
+                ((self.target_framerate.round() as i32) - current_refresh_rate).abs() > 1
             }
             FrameInterpolationMode::Force => { return true; }
         }
@@ -763,22 +774,6 @@ impl GameControl
     {
         let next_scene = Box::new(Scene::<T>::new());
 
-        if self.auto_unload
-        {
-            self.resources.unload_unused();
-        }
-
-        self.next_scene = next_scene;
-        self.next_scene_waiting = true;
-    }
-    
-    pub fn goto_constructed_scene(&mut self, next_scene : Box<dyn BaseScene>)
-    {
-        if self.auto_unload
-        {
-            self.resources.unload_unused();
-        }
-
         self.next_scene = next_scene;
         self.next_scene_waiting = true;
     }
@@ -797,10 +792,15 @@ struct GameloopTimeState
     jiffy_queue : Duration,
     target_frametime : Duration,
     cap_frametime : Duration,
+    frame_start_jiffies : Duration,
     last_display : Instant,
+    #[cfg(feature = "imgui_base")]
+    last_imgui : Instant,
     lag_this_frame : u32,
     lag_this_second : u32,
     thought_yet : bool,
+    lag_recovery : bool,
+    refresh_rate : i32,
     think_timer : GameloopStopwatch,
     draw_timer : GameloopStopwatch,
     wait_timer : GameloopStopwatch,
@@ -813,24 +813,24 @@ impl GameloopTimeState
     {
         let jiffies = Instant::now();
         let last_jiffies = jiffies;
-        let last_fps_jiffies = Instant::now();
-        let jiffy_queue = Duration::new(0, 0);
-        let target_frametime = Duration::new(0, 0);
-        let cap_frametime = Duration::new(0, 0);
-        let last_display = Instant::now();
         
         GameloopTimeState
         {
             jiffies,
             last_jiffies,
-            last_fps_jiffies,
-            jiffy_queue,
-            target_frametime,
-            cap_frametime,
-            last_display,
+            last_fps_jiffies : Instant::now(),
+            jiffy_queue : Duration::new(0, 0),
+            target_frametime : Duration::new(0, 0),
+            cap_frametime : Duration::new(0, 0),
+            frame_start_jiffies : Duration::new(0, 0),
+            last_display : Instant::now(),
+            #[cfg(feature = "imgui_base")]
+            last_imgui : Instant::now(),
             lag_this_frame : 0,
             lag_this_second : 0,
             thought_yet : false,
+            lag_recovery : false,
+            refresh_rate : 0,
             think_timer : GameloopStopwatch::new(),
             draw_timer : GameloopStopwatch::new(),
             wait_timer : GameloopStopwatch::new(),
@@ -840,9 +840,16 @@ impl GameloopTimeState
 
     fn flush(&mut self)
     {
-        while self.jiffy_queue > self.target_frametime
+        if self.jiffy_queue > self.target_frametime * MAX_FRAME_JIFFY_SUBTRACT
         {
-            self.jiffy_queue -= self.target_frametime;
+            self.jiffy_queue = Duration::new(0, 0);
+        }
+        else
+        {
+            while self.jiffy_queue > self.target_frametime
+            {
+                self.jiffy_queue -= self.target_frametime;
+            }
         }
 
         self.new_second();
@@ -860,8 +867,16 @@ impl GameloopTimeState
         self.draw_timer.new_second();
         self.wait_timer.new_second();
         self.present_timer.new_second();
-        
-        self.last_fps_jiffies += Duration::from_secs(1);
+
+        if self.jiffies > self.last_fps_jiffies + Duration::from_secs(MAX_FPS_STAT_CATCHUP)
+        {
+            self.last_fps_jiffies = self.jiffies;
+        }
+        else
+        {
+            self.last_fps_jiffies += Duration::from_secs(1);
+        }
+
         self.lag_this_second = 0;
     }
     
@@ -872,27 +887,45 @@ impl GameloopTimeState
         self.last_jiffies = self.jiffies;
     }
     
-    fn sleep(&mut self, wait_duration : Duration, low_power : bool)
+    fn sleep(&mut self, framerate_wait_target : Duration, low_power : bool)
     {
-        while self.jiffy_queue <= wait_duration
+        while self.jiffy_queue <= framerate_wait_target
         {
             self.update_jiffies();
-            
-            if self.jiffy_queue <= wait_duration - Duration::from_millis(2) || low_power
+
+            let remaining = framerate_wait_target.saturating_sub(self.jiffy_queue);
+
+            // Sleep as long as we reliably can, then busy wait
+            if remaining >= Duration::from_millis(BUSY_WAIT_MARGIN)
             {
-                std::thread::sleep(Duration::from_millis(1));
+                std::thread::sleep(remaining.saturating_sub(Duration::from_millis(BUSY_WAIT_MARGIN)));
+            }
+            else if low_power
+            {
+                std::thread::sleep(remaining);
             }
         }
     }
     
-    fn interpolate_sleep(&mut self, wait_duration : Duration, low_power : bool)
+    fn interpolate_sleep(&mut self, framerate_wait_target : Duration, low_power : bool)
     {
-        while Instant::now() - self.last_display <= wait_duration
+        let mut current_display = Instant::now();
+
+        while current_display - self.last_display <= framerate_wait_target
         {
-            if low_power
+            let remaining = framerate_wait_target.saturating_sub(current_display - self.last_display);
+
+            // Sleep as long as we reliably can, then busy wait
+            if remaining >= Duration::from_millis(BUSY_WAIT_MARGIN)
             {
-                std::thread::sleep(Duration::from_millis(1));
+                std::thread::sleep(remaining.saturating_sub(Duration::from_millis(BUSY_WAIT_MARGIN)));
             }
+            else if low_power
+            {
+                std::thread::sleep(remaining);
+            }
+
+            current_display = Instant::now();
         }
         
         self.last_display = Instant::now();
@@ -1061,7 +1094,16 @@ impl Gameloop
     fn think(&mut self)
     {
         let target_ms = 1000.0 / self.control.target_framerate();
-        let cap_ms = 1000.0 / self.control.frame_interpolation_cap().unwrap_or(1000.0);
+        let cap_ms = match self.control.frame_interpolation_cap()
+        {
+            FrameInterpolationCap::Automatic =>
+            {
+                1000.0 / (self.control.renderer.current_refresh_rate() as f64
+                    * AUTOMATIC_LERP_CAP_MULTIPLIER)
+            }
+            FrameInterpolationCap::Manual(value) => { 1000.0 / value }
+            FrameInterpolationCap::None => { 1.0 }
+        };
         self.time.target_frametime = Duration::from_nanos((target_ms * 1_000_000.0) as u64);
         self.time.cap_frametime = Duration::from_nanos((cap_ms * 1_000_000.0) as u64);
 
@@ -1078,16 +1120,15 @@ impl Gameloop
         
         while self.time.jiffy_queue > self.time.target_frametime
         {
-            // Prevent overflow if CPU cannot process all the frames
-            if (self.time.think_timer.jiffies_this_frame()) > self.time.target_frametime
+            // Don't jump ahead too far after lag ends
+            if self.time.lag_recovery
             {
-                while self.time.jiffy_queue > self.time.target_frametime
+                if self.time.jiffy_queue > self.time.target_frametime * MAX_POST_SLOWDOWN_CATCHUP
                 {
-                    self.time.jiffy_queue -= self.time.target_frametime;
-                    self.time.lag_this_frame += 1;
-                    self.time.lag_this_second += 1;
+                    self.time.jiffy_queue = self.time.target_frametime * MAX_POST_SLOWDOWN_CATCHUP;
                 }
-                break;
+
+                self.time.lag_recovery = false;
             }
 
             self.control.input.update_actions();
@@ -1095,14 +1136,34 @@ impl Gameloop
             self.current_scene.think(&mut self.control);
             
             self.time.think_timer.tic();
-            
-            self.time.jiffy_queue -= self.time.target_frametime;
             self.time.thought_yet = true;
-
             self.control.low_energy_timer += 1;
+
+            // Prevent overflow if CPU cannot process all the frames
+            if (self.time.think_timer.jiffies_this_frame()) > self.time.target_frametime * MAX_LOGIC_FRAME_OVERRUN
+            {
+                while self.time.jiffy_queue > self.time.target_frametime
+                {
+                    self.time.jiffy_queue -= self.time.target_frametime;
+                    self.time.lag_this_frame += 1;
+                    self.time.lag_this_second += 1;
+                    self.time.lag_recovery = true;
+
+                    // Don't spend a really long time if the jiffy queue happens to be really large
+                    if self.time.lag_this_frame > MAX_FRAME_JIFFY_SUBTRACT
+                    {
+                        break;
+                    }
+                }
+                break;
+            }
+
+            self.time.jiffy_queue -= self.time.target_frametime;
         }
         
         self.time.think_timer.end_frame_timing();
+
+        self.time.frame_start_jiffies = self.time.jiffy_queue;
     }
     
     fn draw(&mut self)
@@ -1134,10 +1195,13 @@ impl Gameloop
         {
             let mut interpolation_amount = 0.0;
 
-            if self.control.frame_interpolation_active() && self.time.lag_this_frame == 0 && !self.low_energy_timer_expired()
+            if self.control.frame_interpolation_active() && !self.low_energy_timer_expired()
             {
-                interpolation_amount = (self.time.jiffy_queue.as_nanos() as f64
-                    / self.time.target_frametime.as_nanos() as f64) - 1.0;
+                let jiffy_offset = self.time.jiffy_queue.as_nanos() as f64;
+                let jiffy_target = self.time.target_frametime.as_nanos() as f64;
+
+                interpolation_amount = ((jiffy_offset / jiffy_target) - 1.0)
+                    .clamp(-1.0, 0.0);
             }
 
             self.control.renderer.draw(&mut self.current_scene, &mut self.control.resources, interpolation_amount as f32);
@@ -1150,14 +1214,27 @@ impl Gameloop
         let mut imgui_handler : Option<ImGuiHandler> = None;
 
         let frametime = self.time.target_frametime * self.time.think_timer.tics_this_frame();
-        let frame_delta = frametime.as_nanos() as f64 * MILLISECONDS_PER_NANOSECOND / 1000.0;
         let (width, height) = self.control.renderer.window_size();
+        let scale = self.control.renderer.current_display_scale_factor();
+
+        let frame_delta = match self.control.frame_interpolation_active()
+        {
+            false => { frametime.as_nanos() as f64 * MILLISECONDS_PER_NANOSECOND / 1000.0 }
+            true =>
+            {
+                let now = Instant::now();
+                let delta = now - self.time.last_imgui;
+                self.time.last_imgui = now;
+
+                delta.as_nanos() as f64 * MILLISECONDS_PER_NANOSECOND / 1000.0
+            }
+        };
 
         std::mem::swap(&mut imgui_handler, &mut self.control.imgui_handler);
 
         if let Some(imgui_some) = &mut imgui_handler
         {
-            imgui_some.update_size_and_delta(frame_delta as f32, width as f32, height as f32);
+            imgui_some.update_size_and_delta(frame_delta as f32, width as f32, height as f32, scale);
             self.control.renderer.update_imgui(imgui_some.imgui_mut());
 
             let mut ui = imgui_some.imgui_mut().frame();
@@ -1178,24 +1255,30 @@ impl Gameloop
         true
     }
     
-    fn wait(&mut self)
+    fn wait(&mut self, do_sleep : bool)
     {
+        if !do_sleep
+        {
+            self.time.update_jiffies();
+            return;
+        }
+
         self.time.wait_timer.begin_frame_timing();
 
         let low_energy = self.low_energy_timer_expired();
         
         if !self.control.frame_interpolation_active() || low_energy
         {
-            // Spin until it's time for the next frame
             self.time.sleep(self.time.target_frametime, low_energy);
         }
-        else if self.control.frame_interpolation_cap().is_some()
+        else if self.control.frame_interpolation_cap() != FrameInterpolationCap::None
         {
             self.time.interpolate_sleep(self.time.cap_frametime, low_energy);
             self.time.update_jiffies();
         }
         else
         {
+            self.time.last_display = Instant::now();
             self.time.update_jiffies();
         }
         
@@ -1258,35 +1341,35 @@ impl Gameloop
                 {
                     self.control.focused = false;
                 },
-                WindowEvent::SizeChanged { .. } =>
+                WindowEvent::Moved { .. } | WindowEvent::SizeChanged { .. } =>
                 {
                     self.control.renderer.recalculate_viewport();
                     self.control.input.update_mouse_metrics(&self.control.renderer);
                 }
                 _ => ()
             },
-            Event::KeyDown { scancode, keymod, repeat, .. } =>
+            Event::KeyDown { keycode, keymod, repeat, .. } =>
             {
                 if !repeat && (keymod.contains(KeyboardMod::LALTMOD)
                     || keymod.contains(KeyboardMod::RALTMOD))
-                    && scancode == Some(sdl2::keyboard::Scancode::Return)
+                    && keycode == Some(sdl2::keyboard::Keycode::Return)
                     && !self.kiosk_mode
                 {
                     self.control.renderer.toggle_fullscreen();
                 }
                 else if !repeat
                 {
-                    if let Some(some_scancode) = scancode
+                    if let Some(some_keycode) = keycode
                     {
-                        self.control.input.update_key(some_scancode.name(), true, true);
+                        self.control.input.update_key(&some_keycode.name(), true, true);
                     }
                 }
             },
-            Event::KeyUp { scancode, .. } =>
+            Event::KeyUp { keycode, .. } =>
             {
-                if let Some(some_scancode) = scancode
+                if let Some(some_keycode) = keycode
                 {
-                    self.control.input.update_key(some_scancode.name(), false, true);
+                    self.control.input.update_key(&some_keycode.name(), false, true);
                 }
             },
             Event::MouseMotion { x, y, xrel, yrel, .. } =>
@@ -1352,6 +1435,14 @@ impl Gameloop
     {
         self.control.stats.update_frame_timings(&self.time);
 
+        let current_refresh_rate = self.control.renderer.current_refresh_rate();
+
+        if self.time.refresh_rate != current_refresh_rate
+        {
+            self.time.refresh_rate = current_refresh_rate;
+            info!("Refresh rate: {}hz", current_refresh_rate);
+        }
+
         if self.time.jiffies < self.time.last_fps_jiffies + Duration::from_secs(1)
         {
             return;
@@ -1382,7 +1473,7 @@ impl Gameloop
         #[cfg(debug_assertions)]
         if self.time.lag_this_second > 0
         {
-            warn!("Too much CPU work, game is slowing down!");
+            warn!("Too much logic processing, game is slowing down!");
         }
         
         self.time.new_second();
@@ -1421,14 +1512,7 @@ impl Gameloop
 
         self.think();
         self.draw();
-        if do_sleep
-        {
-            self.wait();
-        }
-        else
-        {
-            self.time.update_jiffies();
-        }
+        self.wait(do_sleep);
         self.present();
 
         self.update_fps();
@@ -1438,6 +1522,11 @@ impl Gameloop
         {
             self.current_scene.end(&mut self.control);
 
+            if self.control.auto_unload
+            {
+                self.control.resources.unload_unused();
+            }
+
             print_log_section("NEW SCENE");
 
             std::mem::swap(&mut self.current_scene, &mut self.control.next_scene);
