@@ -6,21 +6,9 @@ use sdl2::controller::{Button, Axis};
 use sdl2::mouse::MouseButton;
 
 use crate::renderer::Renderer;
+use crate::util::{InputDirection4, InputDirection8};
 
-const ANGLE_UP_RIGHT_RIGHT : f64 = -std::f64::consts::FRAC_PI_8;
-const ANGLE_UP_RIGHT : f64 = -std::f64::consts::FRAC_PI_4;
-const ANGLE_UP_UP_RIGHT : f64 = -std::f64::consts::FRAC_PI_4 - std::f64::consts::FRAC_PI_8;
-const ANGLE_UP_UP_LEFT : f64 = -std::f64::consts::FRAC_PI_2 - std::f64::consts::FRAC_PI_8;
-const ANGLE_UP_LEFT : f64 = -std::f64::consts::FRAC_PI_2 - std::f64::consts::FRAC_PI_4;
-const ANGLE_UP_LEFT_LEFT : f64 = -std::f64::consts::FRAC_PI_2 - std::f64::consts::FRAC_PI_4 - std::f64::consts::FRAC_PI_8;
-const ANGLE_DOWN_LEFT_LEFT : f64 = std::f64::consts::FRAC_PI_2 + std::f64::consts::FRAC_PI_4 + std::f64::consts::FRAC_PI_8;
-const ANGLE_DOWN_LEFT : f64 = std::f64::consts::FRAC_PI_2 + std::f64::consts::FRAC_PI_4;
-const ANGLE_DOWN_DOWN_LEFT : f64 = std::f64::consts::FRAC_PI_2 + std::f64::consts::FRAC_PI_8;
-const ANGLE_DOWN_DOWN_RIGHT : f64 = std::f64::consts::FRAC_PI_4 + std::f64::consts::FRAC_PI_8;
-const ANGLE_DOWN_RIGHT : f64 = std::f64::consts::FRAC_PI_4;
-const ANGLE_DOWN_RIGHT_RIGHT : f64 = std::f64::consts::FRAC_PI_8;
-
-const DIAGONAL_DEAD_ZONE : f64 = std::f64::consts::FRAC_PI_8 / 2.0;
+pub const DIAGONAL_DEAD_ZONE : f64 = std::f64::consts::FRAC_PI_8 / 2.0;
 const DEFAULT_STICK_DEADZONE : f64 = 0.2;
 const DEFAULT_TRIGGER_DEADZONE : f64 = 0.15;
 const AXIS_DEADZONE_MAX : f64 = 0.999;
@@ -104,28 +92,6 @@ enum CompositeAxisData
     }
 }
 
-#[derive(Debug, Clone, Copy, PartialEq, Eq)]
-pub enum InputDirection4
-{
-    Left,
-    Right,
-    Up,
-    Down
-}
-
-#[derive(Debug, Clone, Copy, PartialEq, Eq)]
-pub enum InputDirection8
-{
-    Left,
-    UpLeft,
-    Up,
-    UpRight,
-    Right,
-    DownRight,
-    Down,
-    DownLeft
-}
-
 fn apply_deadzone(deadzone_opt : Option<&f64>, value : f64) -> f64
 {
     if let Some(deadzone) = deadzone_opt
@@ -381,6 +347,21 @@ impl Input
         pos_value - neg_value
     }
 
+    pub fn paired_axis_2d(&self, name_left : &str, name_right : &str, name_up : &str, name_down : &str) -> (f64, f64)
+    {
+        let mut x = self.paired_axis(name_left, name_right);
+        let mut y = self.paired_axis(name_up, name_down);
+        let distance = (x.powi(2) + y.powi(2)).sqrt();
+
+        if distance > 1.0
+        {
+            x *= 1.0 / distance;
+            y *= 1.0 / distance;
+        }
+
+        (x, y)
+    }
+
     pub fn angle_and_distance(&self, name_left : &str, name_right : &str, name_up : &str, name_down : &str) -> (f64, f64)
     {
         let x = self.paired_axis(name_left, name_right);
@@ -401,24 +382,7 @@ impl Input
             return None;
         }
 
-        if angle < ANGLE_UP_LEFT - DIAGONAL_DEAD_ZONE || angle > ANGLE_DOWN_LEFT + DIAGONAL_DEAD_ZONE
-        {
-            return Some(InputDirection4::Left)
-        }
-        else if angle > ANGLE_UP_RIGHT + DIAGONAL_DEAD_ZONE && angle < ANGLE_DOWN_RIGHT - DIAGONAL_DEAD_ZONE
-        {
-            return Some(InputDirection4::Right)
-        }
-        else if angle > ANGLE_UP_LEFT + DIAGONAL_DEAD_ZONE && angle < ANGLE_UP_RIGHT - DIAGONAL_DEAD_ZONE
-        {
-            return Some(InputDirection4::Up)
-        }
-        else if angle > ANGLE_DOWN_RIGHT + DIAGONAL_DEAD_ZONE && angle < ANGLE_DOWN_LEFT - DIAGONAL_DEAD_ZONE
-        {
-            return Some(InputDirection4::Down)
-        }
-
-        None
+        crate::util::angle_to_four_way_direction_deadzone(angle, DIAGONAL_DEAD_ZONE)
     }
 
     pub fn eight_way_direction(&self, name_left : &str, name_right : &str, name_up : &str, name_down : &str) -> Option<InputDirection8>
@@ -430,40 +394,7 @@ impl Input
             return None;
         }
 
-        if angle < ANGLE_UP_LEFT_LEFT || angle >= ANGLE_DOWN_LEFT_LEFT
-        {
-            return Some(InputDirection8::Left)
-        }
-        else if angle >= ANGLE_UP_LEFT_LEFT && angle < ANGLE_UP_UP_LEFT
-        {
-            return Some(InputDirection8::UpLeft)
-        }
-        else if angle >= ANGLE_UP_UP_LEFT && angle < ANGLE_UP_UP_RIGHT
-        {
-            return Some(InputDirection8::Up)
-        }
-        else if angle >= ANGLE_UP_UP_RIGHT && angle < ANGLE_UP_RIGHT_RIGHT
-        {
-            return Some(InputDirection8::UpRight)
-        }
-        else if angle >= ANGLE_UP_RIGHT_RIGHT && angle < ANGLE_DOWN_RIGHT_RIGHT
-        {
-            return Some(InputDirection8::Right)
-        }
-        else if angle >= ANGLE_DOWN_RIGHT_RIGHT && angle < ANGLE_DOWN_DOWN_RIGHT
-        {
-            return Some(InputDirection8::DownRight)
-        }
-        else if angle >= ANGLE_DOWN_DOWN_RIGHT && angle < ANGLE_DOWN_DOWN_LEFT
-        {
-            return Some(InputDirection8::Down)
-        }
-        else if angle >= ANGLE_DOWN_DOWN_LEFT && angle < ANGLE_DOWN_LEFT_LEFT
-        {
-            return Some(InputDirection8::DownLeft)
-        }
-
-        return None;
+        crate::util::angle_to_eight_way_direction(angle)
     }
 
     pub fn cursor_position(&self) -> (f64, f64)
