@@ -3,22 +3,22 @@
 #[cfg(feature = "opengl")] pub mod opengl;
 #[cfg(feature = "vulkan")] pub mod vulkan;
 
-use cgmath::{SquareMatrix, Matrix3};
+use cgmath::{Matrix3, SquareMatrix};
 
 use std::io::Read;
 use std::rc::Rc;
 use std::cell::RefCell;
 
-use serde::{Serialize, Deserialize};
+use serde::{Deserialize, Serialize};
 use itertools::Itertools;
 
-use keeshond_datapack::{DataId, DataObject, DataStore, DataError, PreparedStoreError, DataMultistore, TrustPolicy};
+use keeshond_datapack::{DataError, DataId, DataMultistore, DataObject, DataStore, PreparedStoreError, TrustPolicy};
 use keeshond_datapack::source::Source;
 
 #[cfg(feature = "imgui_base")] use imgui;
 
 use crate::scene::BaseScene;
-use crate::util::{NamedItemStore, Rect, BakedRect};
+use crate::util::{BakedRect, NamedItemStore, Rect};
 #[cfg(feature = "opengl")] use crate::renderer::opengl::GlDrawControl;
 #[cfg(feature = "vulkan")] use crate::renderer::vulkan::VulkanDrawControl;
 
@@ -26,6 +26,14 @@ use crate::util::{NamedItemStore, Rect, BakedRect};
 #[cfg(not(windows))] use sdl2::surface::Surface;
 #[cfg(not(windows))] use sdl2::pixels::PixelFormatEnum;
 
+#[cfg(not(target_os = "macos"))] const BASE_DPI : f32 = 96.0;
+#[cfg(target_os = "macos")] const BASE_DPI : f32 = 72.0;
+
+const MAX_MULTIPLIER_ADJUST : f32 = 0.9;
+
+#[cfg(target_os = "emscripten")] const MAX_RENDERER_SCALE_WIDTH : f32 = 1920.0;
+#[cfg(target_os = "emscripten")] const MAX_RENDERER_SCALE_HEIGHT : f32 = 1080.0;
+
 pub(crate) fn png_to_raw<R : Read>(png_reader : R) -> Result<(Vec<u8>, u32, u32), RendererError>
 {
     let decoder = png::Decoder::new(png_reader);
@@ -39,7 +47,7 @@ pub(crate) fn png_to_raw<R : Read>(png_reader : R) -> Result<(Vec<u8>, u32, u32)
 }
 
 /// Determines how the game is scaled to fit the window or screen
-#[derive(Copy, Clone, PartialEq)]
+#[derive(Copy, Clone, Debug, PartialEq)]
 pub enum ViewportMode
 {
     /// The view is scaled up or down to fit the window in a resolution-independent way
@@ -47,9 +55,54 @@ pub enum ViewportMode
     /// The view is rendered at a fixed size and then scaled up or down with minimal filtering.
     ///  Use for pixel art style games.
     Pixel,
+    /// The viewport base size is the same as the window size, taking display DPI into account.
+    ///  Enlarging the window means you see more of the scene. Use for editors and similar utilities.
+    OneToOneDpiScaled,
     /// The viewport base size is the same as the window size. Enlarging the window means you
-    ///  see more of the scene. Use for editors and similar utilities.
-    OneToOne
+    ///  see more of the scene. You probably want to use OneToOneDpiScaled instead, unless you are
+    ///  doing your own DPI scaling using current_display_scale_factor().
+    OneToOneUnscaled
+}
+
+/// The strategy used to intelligently find the window size based on the game's base resolution
+#[derive(Copy, Clone, Debug, PartialEq)]
+pub enum WindowScaleMode
+{
+    /// The window size is a multiple of the base size, taking the current display DPI into account.
+    ///  If the resulting size would be too big to fit on-screen, uses a smaller size that still fills
+    ///  most of the area.
+    MultiplierDpiScaled(f32),
+    /// The window size is a percentage of the available screen space. Values are between 0.1 and 0.9.
+    ///  Not available on web.
+    #[cfg(not(target_os = "emscripten"))]
+    ScreenPercentage(f32),
+    /// The window size is an integer multiple of the base size, taking the current display DPI into account
+    ///  like MultiplierDpiScaled, but rounding the final size to the closest integer. If the resulting
+    ///  size would be too big to fit on-screen, uses the largest integer scale that fits.
+    PixelMultiplierDpiScaled(u32),
+    /// The window size is the largest integer multiple of the base size that fits on-screen.
+    ///  Not available on web.
+    #[cfg(not(target_os = "emscripten"))]
+    PixelHighestMultiplier,
+    /// The window size is a multiple of the base size, ignoring current display DPI. Not recommended
+    ///  as this can make the window too small for very high DPIs.
+    MultiplierDpiIgnored(f32),
+    /// The window size is an integer multiple of the base size, ignoring current display DPI.
+    ///  This can be useful for getting screenshots or footage at a specific size, but otherwise is
+    ///  not recommended.
+    PixelMultiplierDpiIgnored(u32),
+}
+
+/// Determines the vertical synchronization mode to use
+#[derive(Copy, Clone, Debug, PartialEq)]
+pub enum VsyncMode
+{
+    /// Rendering is not synced to vertical blank, causing tearing. Not recommended.
+    Off,
+    /// Rendering is synced to vertical blank
+    On,
+    /// Rendering is synced to vertical blank unless the framerate drops below the display's refresh rate
+    Adaptive
 }
 
 /// A transformation matrix used for drawing operations
@@ -141,6 +194,93 @@ impl ViewportInfo
     }
 }
 
+fn calculate_display_scale_factor(video_subsystem : &sdl2::VideoSubsystem, display_index : i32) -> f32
+{
+    if let Ok((ddpi, _, _)) = video_subsystem.display_dpi(display_index)
+    {
+        return ddpi / BASE_DPI;
+    }
+
+    1.0
+}
+
+pub(crate) fn calculate_window_size(base_width : f32, base_height : f32, mode : WindowScaleMode, video_subsystem : &sdl2::VideoSubsystem) -> (u32, u32)
+{
+    let display_scale_factor = calculate_display_scale_factor(video_subsystem, 0);
+    let screen_width;
+    let screen_height;
+
+    #[cfg(not(target_os = "emscripten"))]
+    {
+        if let Ok(screen_rect) = video_subsystem.display_usable_bounds(0)
+        {
+            screen_width = screen_rect.width() as f32;
+            screen_height = screen_rect.height() as f32;
+        }
+        else
+        {
+            return (base_width as u32, base_height as u32);
+        }
+    }
+    #[cfg(target_os = "emscripten")]
+    {
+        screen_width = MAX_RENDERER_SCALE_WIDTH;
+        screen_height = MAX_RENDERER_SCALE_HEIGHT;
+    }
+
+    let base_width_dpi = base_width * display_scale_factor;
+    let base_height_dpi = base_height * display_scale_factor;
+    let max_multiplier = (screen_width / base_width).min(screen_height / base_height);
+    let max_multiplier_dpi = (screen_width / base_width_dpi).min(screen_height / base_height_dpi);
+
+    match mode
+    {
+        WindowScaleMode::MultiplierDpiScaled(multiplier) =>
+        {
+            let clamped_multiplier = multiplier.clamp(0.1, max_multiplier_dpi * MAX_MULTIPLIER_ADJUST);
+
+            ((base_width_dpi * clamped_multiplier) as u32,
+                (base_height_dpi * clamped_multiplier) as u32)
+        }
+        #[cfg(not(target_os = "emscripten"))]
+        WindowScaleMode::ScreenPercentage(percentage) =>
+        {
+            ((base_width_dpi * (max_multiplier_dpi * percentage.clamp(0.1, MAX_MULTIPLIER_ADJUST))) as u32,
+                (base_height_dpi * (max_multiplier_dpi * percentage.clamp(0.1, MAX_MULTIPLIER_ADJUST))) as u32)
+        }
+        WindowScaleMode::PixelMultiplierDpiScaled(multiplier) =>
+        {
+            let clamped_multiplier = (multiplier as f32 * display_scale_factor).round()
+                .min((max_multiplier * MAX_MULTIPLIER_ADJUST).floor()).max(1.0);
+
+            ((base_width * (clamped_multiplier)) as u32,
+                (base_height * (clamped_multiplier)) as u32)
+        }
+        #[cfg(not(target_os = "emscripten"))]
+        WindowScaleMode::PixelHighestMultiplier =>
+        {
+            let pixel_highest = (max_multiplier * MAX_MULTIPLIER_ADJUST).floor().max(1.0);
+
+            ((base_width * pixel_highest) as u32,
+                (base_height * pixel_highest) as u32)
+        }
+        WindowScaleMode::MultiplierDpiIgnored(multiplier) =>
+        {
+            let clamped_multiplier = multiplier.clamp(0.1, max_multiplier * MAX_MULTIPLIER_ADJUST);
+
+            ((base_width * clamped_multiplier) as u32,
+                (base_height * clamped_multiplier) as u32)
+        }
+        WindowScaleMode::PixelMultiplierDpiIgnored(multiplier) =>
+        {
+            let clamped_multiplier = multiplier.min((max_multiplier * MAX_MULTIPLIER_ADJUST).floor() as u32).max(1);
+
+            ((base_width * (clamped_multiplier as f32)) as u32,
+                (base_height * (clamped_multiplier as f32)) as u32)
+        }
+    }
+}
+
 pub(crate) fn recalculate_viewport_common(base_width : f32, base_height : f32, width : u32, height : u32, mode : ViewportMode) -> ViewportInfo
 {
     let mut info = ViewportInfo::new();
@@ -150,7 +290,7 @@ pub(crate) fn recalculate_viewport_common(base_width : f32, base_height : f32, w
 
     match mode
     {
-        ViewportMode::Independent | ViewportMode::OneToOne =>
+        ViewportMode::Independent | ViewportMode::OneToOneDpiScaled | ViewportMode::OneToOneUnscaled =>
         {
             info.scale_width = base_width;
             info.scale_height = base_height;
@@ -306,7 +446,7 @@ pub struct RendererNewInfo<'a>
 {
     pub width : u32,
     pub height : u32,
-    pub default_zoom : u32,
+    pub default_window_scale : WindowScaleMode,
     pub texture_filtering : bool,
     pub fullscreen : bool,
     pub kiosk_mode : bool,
@@ -691,9 +831,9 @@ impl DataObject for Sheet
             match metadata_result
             {
                 Err(error) =>
-                    {
-                        return Err(DataError::IoError(error));
-                    },
+                {
+                    return Err(DataError::IoError(error));
+                },
                 _ => {}
             }
 
@@ -841,9 +981,9 @@ impl DataObject for Sheet
         match writer.write_all(json_string.as_bytes())
         {
             Err(error) =>
-                {
-                    return Err(DataError::IoError(error));
-                },
+            {
+                return Err(DataError::IoError(error));
+            },
             _ => {}
         }
 
@@ -1110,6 +1250,14 @@ impl DrawControl for NullDrawControl
     fn update_imgui_renderer(&mut self, imgui : &mut imgui::Context) {}
 }
 
+#[derive(Eq, PartialEq)]
+enum OneToOneMode
+{
+    Off,
+    DpiScaled,
+    Unscaled
+}
+
 /// Represents the renderer's game window
 pub struct Renderer
 {
@@ -1118,7 +1266,7 @@ pub struct Renderer
     video_subsystem : Option<sdl2::VideoSubsystem>,
     base_width : f32,
     base_height : f32,
-    one_to_one_viewport : bool,
+    one_to_one_viewport : OneToOneMode,
     kiosk_mode : bool
 }
 
@@ -1133,13 +1281,17 @@ impl Renderer
         let mut video_subsystem = None;
         let mut fullscreen = false;
         let mut kiosk_mode = false;
-        let mut one_to_one_viewport = false;
+        let mut one_to_one_viewport = OneToOneMode::Off;
         let mut base_width = 100.0;
         let mut base_height = 100.0;
         
         if let Some(mut new_info) = renderer_new_info
         {
-            new_info.default_zoom = new_info.default_zoom.max(1).min(8);
+            #[cfg(windows)]
+            unsafe
+            {
+                winapi::um::winuser::SetProcessDpiAwarenessContext(winapi::shared::windef::DPI_AWARENESS_CONTEXT_PER_MONITOR_AWARE_V2);
+            }
 
             let backend_list = backend_entries_from_string(&new_info.backends);
             
@@ -1158,7 +1310,12 @@ impl Renderer
                         video_subsystem = Some(new_info.video_subsystem);
                         fullscreen = new_info.fullscreen;
                         kiosk_mode = new_info.kiosk_mode;
-                        one_to_one_viewport = new_info.viewport_mode == ViewportMode::OneToOne;
+                        one_to_one_viewport = match new_info.viewport_mode
+                        {
+                            ViewportMode::OneToOneDpiScaled => { OneToOneMode::DpiScaled }
+                            ViewportMode::OneToOneUnscaled => { OneToOneMode::Unscaled }
+                            _ => { OneToOneMode::Off }
+                        };
                         base_width = new_info.width as f32;
                         base_height = new_info.height as f32;
                         
@@ -1290,12 +1447,22 @@ impl Renderer
     
     pub(crate) fn recalculate_viewport(&mut self)
     {
-        if self.one_to_one_viewport
+        match self.one_to_one_viewport
         {
-            let (width, height) = self.window_size();
+            OneToOneMode::Off => {}
+            OneToOneMode::DpiScaled | OneToOneMode::Unscaled =>
+            {
+                let (width, height) = self.window_size();
+                let mut scale = 1.0;
 
-            self.base_width = width as f32;
-            self.base_height = height as f32;
+                if self.one_to_one_viewport == OneToOneMode::DpiScaled
+                {
+                    scale = self.current_display_scale_factor();
+                }
+
+                self.base_width = width as f32 / scale;
+                self.base_height = height as f32 / scale;
+            }
         }
 
         self.control.recalculate_viewport(self.base_width, self.base_height);
@@ -1388,6 +1555,46 @@ impl Renderer
             }
         }
     }
+
+    pub fn vsync(&self) -> VsyncMode
+    {
+        if let Some(video_subsystem) = &self.video_subsystem
+        {
+            match video_subsystem.gl_get_swap_interval()
+            {
+                sdl2::video::SwapInterval::Immediate => { return VsyncMode::Off }
+                sdl2::video::SwapInterval::VSync => { return VsyncMode::On }
+                sdl2::video::SwapInterval::LateSwapTearing => { return VsyncMode::Adaptive }
+            }
+        }
+
+        VsyncMode::On
+    }
+
+    pub fn set_vsync(&mut self, mode : VsyncMode) -> bool
+    {
+        if let Some(video_subsystem) = &self.video_subsystem
+        {
+            let interval = match mode
+            {
+                VsyncMode::Off => { sdl2::video::SwapInterval::Immediate }
+                VsyncMode::On => { sdl2::video::SwapInterval::VSync }
+                VsyncMode::Adaptive => { sdl2::video::SwapInterval::LateSwapTearing }
+            };
+
+            match video_subsystem.gl_set_swap_interval(interval)
+            {
+                Ok(_) => { return true; }
+                Err(error) =>
+                {
+                    error!("Could not set Vsync mode: {}", error);
+                    return false;
+                }
+            }
+        }
+
+        false
+    }
     
     pub fn set_power_management_enabled(&mut self, enabled : bool)
     {
@@ -1424,7 +1631,7 @@ impl Renderer
         false
     }
 
-    pub(crate) fn current_refresh_rate(&self) -> i32
+    pub fn current_refresh_rate(&self) -> i32
     {
         if let Some(window) = &self.window
         {
@@ -1444,4 +1651,22 @@ impl Renderer
 
         0
     }
+
+    pub fn current_display_scale_factor(&self) -> f32
+    {
+        if let Some(window) = &self.window
+        {
+            let window_borrow = window.borrow();
+
+            if let Ok(index) = window_borrow.display_index()
+            {
+                if let Some(video_subsystem) = &self.video_subsystem
+                {
+                    return calculate_display_scale_factor(video_subsystem, index);
+                }
+            }
+        }
+
+        1.0
+    }
 }
