@@ -20,7 +20,7 @@ use crate::renderer::{DrawControl, DrawControlResult, DrawTransform, DrawSpriteR
 use crate::util::BakedRect;
 
 const INSTANCES_MAX : usize = 8192;
-const INSTANCE_BUFFER_MAX: usize = 16;
+const INSTANCE_BUFFER_MAX: usize = 32;
 
 const UNBORKED_IDENTITY_TRANSFORM : [f32; 9] = [2.0, 0.0, 0.0, 0.0, -2.0, 0.0, -1.0, 1.0, 1.0];
 const UNBORKED_UPSCALE_TRANSFORM : [f32; 9] = [2.0, 0.0, 0.0, 0.0, 2.0, 0.0, -1.0, -1.0, 1.0];
@@ -464,7 +464,7 @@ impl GlDrawControl
                 
                 match self.viewport_mode
                 {
-                    ViewportMode::Independent | ViewportMode::OneToOne =>
+                    ViewportMode::Independent | ViewportMode::OneToOneDpiScaled | ViewportMode::OneToOneUnscaled =>
                     {
                         let params = glium::DrawParameters
                         {
@@ -565,8 +565,13 @@ impl DrawControl for GlDrawControl
     fn new(renderer_new_info : &mut RendererNewInfo) -> DrawControlResult where Self : Sized
     {
         let video_subsystem = &renderer_new_info.video_subsystem;
-        let mut window_builder = video_subsystem.window("", renderer_new_info.width * renderer_new_info.default_zoom,
-        renderer_new_info.height * renderer_new_info.default_zoom);
+
+        let (width, height) = crate::renderer::calculate_window_size(renderer_new_info.width as f32,
+                                                                     renderer_new_info.height as f32,
+                                                                     renderer_new_info.default_window_scale,
+                                                                     video_subsystem);
+
+        let mut window_builder = video_subsystem.window("", width, height);
         
         window_builder.hidden().position_centered().resizable().allow_highdpi();
 
@@ -588,7 +593,7 @@ impl DrawControl for GlDrawControl
         
         match renderer_new_info.viewport_mode
         {
-            ViewportMode::Independent | ViewportMode::OneToOne => (),
+            ViewportMode::Independent | ViewportMode::OneToOneDpiScaled | ViewportMode::OneToOneUnscaled => (),
             ViewportMode::Pixel =>
             {
                 pixel_texture = Some(try_or_else!(Texture2d::empty_with_format(
@@ -915,7 +920,7 @@ impl DrawControl for GlDrawControl
         
         match self.viewport_mode
         {
-            ViewportMode::Independent | ViewportMode::OneToOne =>
+            ViewportMode::Independent | ViewportMode::OneToOneDpiScaled | ViewportMode::OneToOneUnscaled =>
             {
                 let (x, y) = self.viewport_offset;
                 
@@ -948,7 +953,7 @@ impl DrawControl for GlDrawControl
         {
             match self.viewport_mode
             {
-                ViewportMode::Independent | ViewportMode::OneToOne => (),
+                ViewportMode::Independent | ViewportMode::OneToOneDpiScaled | ViewportMode::OneToOneUnscaled => (),
                 ViewportMode::Pixel =>
                 {
                     // Low-res pixel to high-res pixel
