@@ -654,11 +654,11 @@ impl DrawControl for VulkanDrawControl
 
         match self.viewport_mode
         {
-            ViewportMode::Independent | ViewportMode::OneToOne =>
+            ViewportMode::Independent | ViewportMode::OneToOneDpiScaled | ViewportMode::OneToOneUnscaled =>
             {
                 view_transform.scale(1.0 / self.scale_width, 1.0 / self.scale_height);
                 view_transform.translate(self.scale_offset_x, self.scale_offset_y);
-            },
+            }
             ViewportMode::Pixel =>
             {
                 unimplemented!();
