@@ -36,6 +36,7 @@ use std::marker::PhantomData;
 
 type ComponentTypeMax = U1024;
 
+const MAX_RESERVE_CAPACITY_HINT : usize = 2097152;
 
 #[macro_export] macro_rules! spawnables
 {
@@ -114,7 +115,7 @@ type ComponentTypeMax = U1024;
 ////////////////////////////////////////////////////////////
 
 /// A handle to a living object in a [Scene]
-#[derive(Debug, PartialEq, Eq, Hash, Serialize, Deserialize, Clone)]
+#[derive(Debug, PartialEq, Eq, PartialOrd, Ord, Hash, Serialize, Deserialize, Clone)]
 pub struct Entity
 {
     #[serde(rename = "i")]
@@ -204,6 +205,13 @@ pub trait Component : Downcast
 }
 impl_downcast!(Component);
 
+// A struct that contains both an entity and a component, for passing entity-component pairs around
+pub struct EntityComponent<C : Component + 'static>
+{
+    pub entity : Entity,
+    pub component : C
+}
+
 pub struct ThinkerArgs<'a, T : SceneType + 'static>
 {
     pub components : &'a mut ComponentControl,
@@ -759,11 +767,16 @@ impl<C : Component + 'static> ComponentStore<C>
     
     pub fn sort<F : FnMut(&C, &C) -> Ordering>(&mut self, mut compare : F)
     {
-        self.component_list.sort_by(|a, b|
+        self.component_list.sort_unstable_by(|a, b|
         {
-            let (component_a, _) = a;
-            let (component_b, _) = b;
-            compare(component_a, component_b)
+            let (component_a, entity_a) = a;
+            let (component_b, entity_b) = b;
+
+            match compare(component_a, component_b)
+            {
+                Ordering::Equal => { entity_a.cmp(entity_b) },
+                other => { other }
+            }
         });
         
         for i in 0..self.component_list.len()
@@ -812,6 +825,18 @@ impl<C : Component + 'static> ComponentStore<C>
         
         self.singleton_item.as_mut().unwrap()
     }
+
+    pub fn reserve_capacity(&mut self, additional : usize)
+    {
+        self.component_list.reserve(additional);
+        self.entity_mapping.reserve(additional);
+    }
+
+    pub fn shrink_to_fit(&mut self)
+    {
+        self.component_list.shrink_to_fit();
+        self.entity_mapping.shrink_to_fit();
+    }
 }
 
 impl<C : Component + 'static> ComponentStoreBase for ComponentStore<C>
@@ -1171,13 +1196,107 @@ impl ComponentControl
 
 ////////////////////////////////////////////////////////////
 
+trait DeferredComponentListBase : Downcast
+{
+    fn add_to_scene(&mut self, scene_components : &mut SceneComponents);
+}
+impl_downcast!(DeferredComponentListBase);
+
+struct DeferredComponentList<C : Component + 'static>
+{
+    list : Vec<EntityComponent<C>>
+}
+
+impl<C : Component + 'static> DeferredComponentList<C>
+{
+    fn new(reserve_capacity : usize) -> DeferredComponentList<C>
+    {
+        DeferredComponentList
+        {
+            list : Vec::with_capacity(reserve_capacity)
+        }
+    }
+}
+
+impl<C : Component + 'static> DeferredComponentListBase for DeferredComponentList<C>
+{
+    fn add_to_scene(&mut self, scene_components : &mut SceneComponents)
+    {
+        let mut store = scene_components.control.store_mut::<C>();
+
+        if self.list.len() > store.count()
+        {
+            store.reserve_capacity(self.list.len());
+        }
+
+        std::mem::drop(store);
+
+        let mut list = Vec::<EntityComponent<C>>::new();
+        std::mem::swap(&mut list, &mut self.list);
+
+        scene_components.add_multiple_components(list);
+    }
+}
+
+
+struct DeferredComponentAdds
+{
+    lists : Vec<Box<dyn DeferredComponentListBase>>,
+    type_map : HashMap<TypeId, usize, BuildHasherDefault<FxHasher>>,
+    reserve_capacity : usize
+}
+
+impl DeferredComponentAdds
+{
+    fn new() -> DeferredComponentAdds
+    {
+        DeferredComponentAdds
+        {
+            lists : Vec::new(),
+            type_map : HashMap::with_hasher(BuildHasherDefault::<FxHasher>::default()),
+            reserve_capacity : 0
+        }
+    }
+
+    fn push<C : Component + 'static>(&mut self, entity : Entity, component : C)
+    {
+        if let Some(index) = self.type_map.get(&TypeId::of::<C>())
+        {
+            let list_downcast = self.lists[*index].downcast_mut::<DeferredComponentList<C>>().expect("Bad deferred list downcast!");
+
+            list_downcast.list.push(EntityComponent
+            {
+                entity,
+                component
+            });
+        }
+        else
+        {
+            self.type_map.insert(TypeId::of::<C>(), self.lists.len());
+            self.lists.push(Box::new(DeferredComponentList::<C>::new(self.reserve_capacity.min(MAX_RESERVE_CAPACITY_HINT))));
+
+            self.push(entity, component);
+        }
+    }
+
+    fn clear(&mut self)
+    {
+        self.type_map.clear();
+        self.lists.clear();
+        self.reserve_capacity = 0;
+    }
+
+    fn is_empty(&self) -> bool
+    {
+        self.lists.is_empty()
+    }
+}
 
 pub struct SceneControl<T : SceneType + 'static>
 {
     unused_pool : VecDeque<Entity>,
     deferred_adds : Vec<Entity>,
     deferred_removals : Vec<Entity>,
-    deferred_component_adds : Vec<(TypeId, Entity, Box<dyn Component>)>,
     deferred_component_removals : Vec<(TypeId, Entity)>,
     deferred_callback_adds : Vec<(TypeId, ComponentCallbacks)>,
     deferred_flag : bool,
@@ -1195,7 +1314,6 @@ impl<T : SceneType + 'static> SceneControl<T>
             unused_pool : VecDeque::new(),
             deferred_adds : Vec::new(),
             deferred_removals : Vec::new(),
-            deferred_component_adds : Vec::new(),
             deferred_component_removals : Vec::new(),
             deferred_callback_adds : Vec::new(),
             deferred_flag : false,
@@ -1223,9 +1341,7 @@ impl<T : SceneType + 'static> SceneControl<T>
     
     pub fn add_component_later<C : Component + 'static>(&mut self, entity : &Entity, component : C)
     {
-        self.deferred_component_adds.push(
-            (TypeId::of::<C>(), Entity::new(entity.id, entity.generation), Box::new(component)));
-        
+        self.spawn_control.deferred_adds.push(entity.clone(), component);
         self.create_component_factory::<C>();
         
         self.deferred_flag = true;
@@ -1252,17 +1368,12 @@ impl<T : SceneType + 'static> SceneControl<T>
     pub fn spawn_later_with(&mut self, spawnable_id : T::SpawnableIdType, game : &mut GameControl,
                             x : f64, y : f64, args : &DynArgList) -> Entity
     {
-        let mut components = Vec::new();
         let entity = self.get_next_entity();
 
+        self.spawn_control.entity = entity.clone();
         self.scene_type.spawn(&mut self.spawn_control, spawnable_id, game, x, y, args);
-        
-        std::mem::swap(&mut components, &mut self.spawn_control.components);
+
         self.deferred_adds.push(entity.clone());
-        for component in components.drain(..)
-        {
-            self.deferred_component_adds.push(((&*component).type_id(), entity.clone(), component));
-        }
         self.deferred_flag = true;
 
         entity
@@ -1291,6 +1402,11 @@ impl<T : SceneType + 'static> SceneControl<T>
         
         entity
     }
+
+    pub fn deferred_reserve_capacity_hint(&mut self, additional : usize)
+    {
+        self.spawn_control.deferred_adds.reserve_capacity = additional;
+    }
 }
 
 
@@ -1300,7 +1416,8 @@ impl<T : SceneType + 'static> SceneControl<T>
 pub struct SpawnControl
 {
     store_factories : HashMap<TypeId, Box<dyn Fn() -> Box<dyn ComponentStoreBase>>, BuildHasherDefault<FxHasher>>,
-    components : Vec<Box<dyn Component>>
+    deferred_adds : DeferredComponentAdds,
+    entity : Entity
 }
 
 impl SpawnControl
@@ -1310,16 +1427,17 @@ impl SpawnControl
         SpawnControl
         {
             store_factories : HashMap::with_capacity_and_hasher(ComponentTypeMax::to_usize(),
-                                        BuildHasherDefault::<FxHasher>::default()),
-            components : Vec::new()
+                                                                BuildHasherDefault::<FxHasher>::default()),
+            deferred_adds : DeferredComponentAdds::new(),
+            entity : Entity::null()
         }
     }
-    
+
     pub fn with<C : Component + 'static>(&mut self, component : C)
     {
         self.create_component_factory::<C>();
-        
-        self.components.push(Box::new(component));
+
+        self.deferred_adds.push(self.entity.clone(), component);
     }
 
     #[inline(always)]
@@ -1449,7 +1567,6 @@ impl SceneType for NullSceneType
 }
 
 pub type SimpleScene = Scene::<NullSceneType>;
-pub type SimpleSceneControl = SceneControl::<NullSceneType>;
 
 
 /// A base trait for scenes that can be run from within the gameloop
@@ -1480,21 +1597,175 @@ struct EntityRecord
     generation : u64
 }
 
-/// A world in which [Entities](Entity), [Components](Component), and [ThinkerSystems](ThinkerSystem)/[DrawerSystems](DrawerSystem) reside.
-pub struct Scene<T : SceneType + 'static>
+struct SceneComponents
 {
-    components : ComponentControl,
-    control : SceneControl<T>,
+    control : ComponentControl,
     entity_records : Vec<EntityRecord>,
-    thinker_systems : Vec<Box<dyn ThinkerSystem<T>>>,
-    drawer_systems : Vec<Box<dyn DrawerSystem>>,
-    #[cfg(feature = "imgui_base")]
-    imgui_system : Box<dyn ImGuiSystem<T>>,
     num_entities : usize,
     num_components : usize,
     component_callbacks : GenericArray<ComponentCallbacks, ComponentTypeMax>,
     need_purge : bool,
     stores_to_purge : BitArraySet<u32, ComponentTypeMax>,
+}
+
+impl SceneComponents
+{
+    fn new() -> SceneComponents
+    {
+        let component_callbacks = GenericArray::<ComponentCallbacks, ComponentTypeMax>::generate(|_| ComponentCallbacks::new());
+
+        SceneComponents
+        {
+            control : ComponentControl::new(),
+            entity_records : Vec::new(),
+            num_entities : 0,
+            num_components : 0,
+            component_callbacks,
+            need_purge : false,
+            stores_to_purge : BitArraySet::new()
+        }
+    }
+
+    fn add_component<C : Component + 'static>(&mut self, entity : &Entity, component : C) -> bool
+    {
+        let wanted_type = TypeId::of::<C>();
+
+        if entity.id >= self.entity_records.len()
+        {
+            return false;
+        }
+
+        if !self.control.component_type_registered(wanted_type)
+        {
+            self.control.register_component_type::<C>();
+        }
+
+        let record = &mut self.entity_records[entity.id];
+
+        if entity.generation != record.generation
+        {
+            return false;
+        }
+
+        let data = self.control.data.try_borrow_mut().expect("ComponentStore data already in use!");
+        let index = data.component_id_to_index[&wanted_type];
+
+        let mut component_store = RefMut::map(self.control.component_stores[index].borrow_mut(),
+                                              |b| b.downcast_mut::<ComponentStore<C>>().expect("ComponentStore borrow failed!"));
+
+        let result = component_store.add_entity(entity, component);
+
+        std::mem::drop(data);
+        std::mem::drop(component_store);
+
+        if result
+        {
+            self.num_components += 1;
+            record.component_bits.insert(index);
+
+            self.component_callbacks[index].call_add_component_callback(entity, &mut self.control);
+        }
+
+        return result;
+    }
+
+    fn add_multiple_components<C : Component + 'static>(&mut self, mut list : Vec<EntityComponent<C>>)
+    {
+        let wanted_type = TypeId::of::<C>();
+
+        if !self.control.component_type_registered(wanted_type)
+        {
+            self.control.register_component_type::<C>();
+        }
+
+        let data = self.control.data.try_borrow_mut().expect("ComponentStore data already in use!");
+        let component_index = data.component_id_to_index[&wanted_type];
+
+        std::mem::drop(data);
+
+        for item in list.drain(..)
+        {
+            if item.entity.id >= self.entity_records.len()
+            {
+                continue;
+            }
+
+            let record = &mut self.entity_records[item.entity.id];
+
+            if item.entity.generation != record.generation
+            {
+                continue;
+            }
+
+            let mut component_store = RefMut::map(self.control.component_stores[component_index].borrow_mut(),
+                                                  |b| b.downcast_mut::<ComponentStore<C>>().expect("ComponentStore borrow failed!"));
+
+            let result = component_store.add_entity(&item.entity, item.component);
+
+            std::mem::drop(component_store);
+
+            if result
+            {
+                self.num_components += 1;
+                record.component_bits.insert(component_index);
+
+                self.component_callbacks[component_index].call_add_component_callback(&item.entity, &mut self.control);
+            }
+        }
+    }
+
+    fn remove_component(&mut self, entity : &Entity, component_type : TypeId) -> bool
+    {
+        let data = self.control.data.try_borrow_mut().expect("ComponentStore data already in use!");
+
+        if entity.id >= self.entity_records.len() || !data.component_id_to_index.contains_key(&component_type)
+        {
+            return false;
+        }
+
+        let record = &mut self.entity_records[entity.id];
+
+        if entity.generation != record.generation
+        {
+            return false;
+        }
+
+        let index = data.component_id_to_index[&component_type];
+
+        let mut component_store = self.control.component_stores[index].borrow_mut();
+        let result = component_store.preview_remove_entity(entity);
+
+        std::mem::drop(data);
+
+        if result
+        {
+            component_store.do_remove_entity(entity);
+
+            std::mem::drop(component_store);
+
+            self.num_components -= 1;
+            record.component_bits.remove(index);
+            self.stores_to_purge.insert(index);
+            self.need_purge = true;
+
+            self.component_callbacks[index].call_remove_component_callback(entity, &mut self.control);
+        }
+
+        return result;
+    }
+}
+
+/// A world in which [Entities](Entity), [Components](Component), and [ThinkerSystems](ThinkerSystem)/[DrawerSystems](DrawerSystem) reside.
+///  You will normally only be working with this API if you are directly managing a Scene within a Scene.
+///  Usually you will be using SceneControl and ComponentControl instead.
+pub struct Scene<T : SceneType + 'static>
+{
+    components : SceneComponents,
+    control : SceneControl<T>,
+    thinker_systems : Vec<Box<dyn ThinkerSystem<T>>>,
+    drawer_systems : Vec<Box<dyn DrawerSystem>>,
+    #[cfg(feature = "imgui_base")]
+    imgui_system : Box<dyn ImGuiSystem<T>>,
     init_done : bool,
     deinit_done : bool
 }
@@ -1521,7 +1792,7 @@ impl<T : SceneType + 'static> BaseScene for Scene<T>
         {
             system.think(ThinkerArgs
             {
-                components : &mut self.components, scene : &mut self.control, game
+                components : &mut self.components.control, scene : &mut self.control, game
             });
             
             self.do_deferred_actions();
@@ -1559,7 +1830,7 @@ impl<T : SceneType + 'static> BaseScene for Scene<T>
         }
         for (type_id, factory) in config.store_factories
         {
-            self.components.register_boxed_component_type(type_id, factory());
+            self.components.control.register_boxed_component_type(type_id, factory());
         }
 
         let mut list = Vec::new();
@@ -1568,14 +1839,14 @@ impl<T : SceneType + 'static> BaseScene for Scene<T>
 
         for system in list.iter_mut()
         {
-            system.start(ThinkerArgs { components : &mut self.components, scene : &mut self.control, game });
+            system.start(ThinkerArgs { components : &mut self.components.control, scene : &mut self.control, game });
 
             self.do_deferred_actions();
         }
 
         for system in list.iter_mut()
         {
-            system.start_late(ThinkerArgs { components : &mut self.components, scene : &mut self.control, game });
+            system.start_late(ThinkerArgs { components : &mut self.components.control, scene : &mut self.control, game });
 
             self.do_deferred_actions();
         }
@@ -1586,7 +1857,7 @@ impl<T : SceneType + 'static> BaseScene for Scene<T>
         {
             self.imgui_system.start(ThinkerArgs
             {
-                components : &mut self.components, scene : &mut self.control, game
+                components : &mut self.components.control, scene : &mut self.control, game
             });
 
             self.do_deferred_actions();
@@ -1617,7 +1888,7 @@ impl<T : SceneType + 'static> BaseScene for Scene<T>
         {
             system.end(ThinkerArgs
             {
-                components : &mut self.components, scene : &mut self.control, game
+                components : &mut self.components.control, scene : &mut self.control, game
             });
             
             self.do_deferred_actions();
@@ -1629,7 +1900,7 @@ impl<T : SceneType + 'static> BaseScene for Scene<T>
         {
             self.imgui_system.end(ThinkerArgs
             {
-                components : &mut self.components, scene : &mut self.control, game
+                components : &mut self.components.control, scene : &mut self.control, game
             });
 
             self.do_deferred_actions();
@@ -1652,17 +1923,17 @@ impl<T : SceneType + 'static> BaseScene for Scene<T>
         let mut list = Vec::new();
         
         std::mem::swap(&mut list, &mut self.drawer_systems);
-        self.components.locked = true;
+        self.components.control.locked = true;
         
         for system in list.iter()
         {
             system.draw(DrawerArgs
             {
-                components : &self.components, resources, drawing, transform, interpolation
+                components : &self.components.control, resources, drawing, transform, interpolation
             });
         }
 
-        self.components.locked = false;
+        self.components.control.locked = false;
         std::mem::swap(&mut list, &mut self.drawer_systems);
     }
     
@@ -1676,20 +1947,20 @@ impl<T : SceneType + 'static> BaseScene for Scene<T>
         
         self.imgui_system.imgui_think(ui, ThinkerArgs
         {
-            components : &mut self.components, scene : &mut self.control, game
+            components : &mut self.components.control, scene : &mut self.control, game
         });
     }
     
     /// Returns the number of [Entities](Entity) in the scene
     fn entity_count(&self) -> usize
     {
-        self.num_entities
+        self.components.num_entities
     }
     
     /// Returns the number of [Components](Component) in the scene
     fn component_count(&self) -> usize
     {
-        self.num_components
+        self.components.num_components
     }
 }
 
@@ -1698,22 +1969,14 @@ impl<T : SceneType + 'static> Scene<T>
     /// Creates a new [Scene]
     pub fn new() -> Self
     {
-        let component_callbacks = GenericArray::<ComponentCallbacks, ComponentTypeMax>::generate(|_| ComponentCallbacks::new());
-
         Scene
         {
-            components : ComponentControl::new(),
+            components : SceneComponents::new(),
             control : SceneControl::<T>::new(),
-            entity_records : Vec::new(),
             thinker_systems : Vec::new(),
             drawer_systems : Vec::new(),
             #[cfg(feature = "imgui_base")]
             imgui_system : Box::new(NullImGuiSystem { _phantom : PhantomData }),
-            num_entities : 0,
-            num_components : 0,
-            component_callbacks,
-            need_purge : false,
-            stores_to_purge : BitArraySet::new(),
             init_done : false,
             deinit_done : false,
         }
@@ -1723,146 +1986,45 @@ impl<T : SceneType + 'static> Scene<T>
     /// normally don't need to call this yourself.
     pub fn component_control(&self) -> &ComponentControl
     {
-        &self.components
+        &self.components.control
     }
     
     /// Returns a mutable reference to the control used to access [ComponentStores](ComponentStore). You
     /// normally don't need to call this yourself.
     pub fn component_control_mut(&mut self) -> &mut ComponentControl
     {
-        &mut self.components
+        &mut self.components.control
     }
     
     pub fn register_component_type<C : Component + 'static>(&mut self) -> bool
     {
-        self.components.register_component_type::<C>()
+        self.components.control.register_component_type::<C>()
     }
     
     /// Attaches a [Component] to the given [Entity]. Returns true if successful.
     pub fn add_component<C : Component + 'static>(&mut self, entity : &Entity, component : C) -> bool
     {
-        let wanted_type = TypeId::of::<C>();
-        
-        if entity.id >= self.entity_records.len()
-        {
-            return false;
-        }
-        
-        if !self.components.component_type_registered(wanted_type)
-        {
-            self.components.register_component_type::<C>();
-        }
-
-        let record = &mut self.entity_records[entity.id];
-
-        if entity.generation != record.generation
-        {
-            return false;
-        }
-
-        let data = self.components.data.try_borrow_mut().expect("ComponentStore data already in use!");
-        let index = data.component_id_to_index[&wanted_type];
-
-        let mut component_store = RefMut::map(self.components.component_stores[index].borrow_mut(),
-                    |b| b.downcast_mut::<ComponentStore<C>>().expect("ComponentStore borrow failed!"));
-
-        let result = component_store.add_entity(entity, component);
-
-        std::mem::drop(data);
-        std::mem::drop(component_store);
-
-        if result
-        {
-            self.num_components += 1;
-            record.component_bits.insert(index);
-
-            self.component_callbacks[index].call_add_component_callback(entity, &mut self.components);
-        }
-
-        return result;
+        self.components.add_component(entity, component)
     }
-    
-    fn add_component_boxed(&mut self, entity : &Entity, component : Box<dyn Component>, component_type : TypeId) -> bool
+
+    /// Attaches multiple [Components](Component) to their given [Entities](Entity). Calling this
+    ///  should be faster than calling add_component() multiple times.
+    pub fn add_multiple_components<C : Component + 'static>(&mut self, list : Vec<EntityComponent<C>>)
     {
-        if entity.id >= self.entity_records.len()
-        {
-            return false;
-        }
-        
-        self.ensure_boxed_component_registered(component_type);
-        
-        let record = &mut self.entity_records[entity.id];
-
-        if entity.generation != record.generation
-        {
-            return false;
-        }
-
-        let data = self.components.data.try_borrow_mut().expect("ComponentStore data already in use!");
-        let index = data.component_id_to_index[&component_type];
-
-        let mut component_store = self.components.component_stores[index].borrow_mut();
-        let result = component_store.add_entity_boxed(entity, component);
-
-        std::mem::drop(data);
-        std::mem::drop(component_store);
-
-        if result
-        {
-            self.num_components += 1;
-            record.component_bits.insert(index);
-
-            self.component_callbacks[index].call_add_component_callback(entity, &mut self.components);
-        }
-
-        return result;
+        self.components.add_multiple_components(list)
     }
     
     /// Detaches a [Component] from the given [Entity]. Returns true if successful.
     pub fn remove_component<C : Component + 'static>(&mut self, entity : &Entity) -> bool
     {
-        self.remove_component_internal(entity, TypeId::of::<C>())
-    }
-    
-    fn remove_component_internal(&mut self, entity : &Entity, component_type : TypeId) -> bool
-    {
-        let data = self.components.data.try_borrow_mut().expect("ComponentStore data already in use!");
-
-        if entity.id >= self.entity_records.len() || !data.component_id_to_index.contains_key(&component_type)
-        {
-            return false;
-        }
-        
-        let record = &mut self.entity_records[entity.id];
-
-        if entity.generation != record.generation
-        {
-            return false;
-        }
-
-        let index = data.component_id_to_index[&component_type];
-
-        let mut component_store = self.components.component_stores[index].borrow_mut();
-        let result = component_store.preview_remove_entity(entity);
-
-        std::mem::drop(data);
+        let result = self.components.remove_component(entity, TypeId::of::<C>());
 
         if result
         {
-            component_store.do_remove_entity(entity);
-
-            std::mem::drop(component_store);
-
-            self.num_components -= 1;
-            record.component_bits.remove(index);
-            self.stores_to_purge.insert(index);
-            self.need_purge = true;
             self.control.deferred_flag = true;
-
-            self.component_callbacks[index].call_remove_component_callback(entity, &mut self.components);
         }
 
-        return result;
+        result
     }
     
     /// Creates a new [Entity] within the scene, returning its handler.
@@ -1879,16 +2041,16 @@ impl<T : SceneType + 'static> Scene<T>
     {
         debug_assert_ne!(entity.generation, 0);
 
-        if entity.id < self.entity_records.len()
+        if entity.id < self.components.entity_records.len()
         {
-            let record = &mut self.entity_records[entity.id];
+            let record = &mut self.components.entity_records[entity.id];
 
             record.component_bits = BitArraySet::new();
             record.generation = entity.generation;
         }
-        else if entity.id == self.entity_records.len()
+        else if entity.id == self.components.entity_records.len()
         {
-            self.entity_records.insert(entity.id, EntityRecord
+            self.components.entity_records.insert(entity.id, EntityRecord
             {
                 component_bits : BitArraySet::new(),
                 generation : entity.generation
@@ -1896,21 +2058,21 @@ impl<T : SceneType + 'static> Scene<T>
         }
         else
         {
-            panic!("Unexpected entity ID number. Got {}, expected {} or lower", entity.id, self.entity_records.len());
+            panic!("Unexpected entity ID number. Got {}, expected {} or lower", entity.id, self.components.entity_records.len());
         }
         
-        self.num_entities += 1;
+        self.components.num_entities += 1;
     }
     
     /// Removes the [Entity] with the given handle from the scene. Returns true if successful.
     pub fn remove_entity(&mut self, entity : &Entity) -> bool
     {
-        if entity.id >= self.entity_records.len()
+        if entity.id >= self.components.entity_records.len()
         {
             return false;
         }
         
-        let record = &mut self.entity_records[entity.id];
+        let record = &mut self.components.entity_records[entity.id];
 
         if entity.generation != record.generation
         {
@@ -1919,16 +2081,16 @@ impl<T : SceneType + 'static> Scene<T>
 
         for component_index in &record.component_bits
         {
-            if self.components.component_stores[component_index].borrow_mut().preview_remove_entity(&entity)
+            if self.components.control.component_stores[component_index].borrow_mut().preview_remove_entity(&entity)
             {
-                self.component_callbacks[component_index].call_remove_component_callback(entity, &mut self.components);
+                self.components.component_callbacks[component_index].call_remove_component_callback(entity, &mut self.components.control);
 
-                let mut component_store = self.components.component_stores[component_index].borrow_mut();
+                let mut component_store = self.components.control.component_stores[component_index].borrow_mut();
 
                 component_store.do_remove_entity(&entity);
-                self.num_components -= 1;
-                self.stores_to_purge.insert(component_index);
-                self.need_purge = true;
+                self.components.num_components -= 1;
+                self.components.stores_to_purge.insert(component_index);
+                self.components.need_purge = true;
                 self.control.deferred_flag = true;
             }
         }
@@ -1945,7 +2107,7 @@ impl<T : SceneType + 'static> Scene<T>
 
         self.control.unused_pool.push_back(Entity::new(entity.id, next_generation));
 
-        self.num_entities -= 1;
+        self.components.num_entities -= 1;
 
         return true;
     }
@@ -1953,12 +2115,12 @@ impl<T : SceneType + 'static> Scene<T>
     /// Returns true if the scene has the given [Entity].
     pub fn has_entity(&self, entity : &Entity) -> bool
     {
-        if entity.id >= self.entity_records.len()
+        if entity.id >= self.components.entity_records.len()
         {
             return false;
         }
         
-        let record = &self.entity_records[entity.id];
+        let record = &self.components.entity_records[entity.id];
 
         record.generation > 0 && entity.generation == record.generation
     }
@@ -1972,39 +2134,38 @@ impl<T : SceneType + 'static> Scene<T>
     pub fn spawn_with(&mut self, spawnable_id : T::SpawnableIdType, game : &mut GameControl,
         x : f64, y : f64, args : &DynArgList)
     {
-        self.control.scene_type.spawn(&mut self.control.spawn_control, spawnable_id, game, x, y, args);
-        
         let entity = self.add_entity();
 
-        let mut components = Vec::new();
-        std::mem::swap(&mut components, &mut self.control.spawn_control.components);
+        self.control.spawn_control.entity = entity.clone();
+        self.control.scene_type.spawn(&mut self.control.spawn_control, spawnable_id, game, x, y, args);
 
-        for component in components.drain(..)
+        for list in &mut self.control.spawn_control.deferred_adds.lists
         {
-            let component_type = (*component).type_id();
-            self.add_component_boxed(&entity, component, component_type);
+            list.add_to_scene(&mut self.components);
         }
+
+        self.control.spawn_control.deferred_adds.clear();
     }
 
     fn push_callbacks_internal(&mut self, component_type : TypeId, callbacks : ComponentCallbacks)
     {
         self.ensure_boxed_component_registered(component_type);
 
-        let data = self.components.data.try_borrow_mut().expect("ComponentStore data already in use!");
+        let data = self.components.control.data.try_borrow_mut().expect("ComponentStore data already in use!");
         let index = data.component_id_to_index.get(&component_type).expect("Unregistered Component type for callback!");
 
-        self.component_callbacks[*index].append(callbacks);
+        self.components.component_callbacks[*index].append(callbacks);
     }
 
     #[inline(always)]
     fn ensure_boxed_component_registered(&mut self, component_type : TypeId)
     {
-        if !self.components.component_type_registered(component_type)
+        if !self.components.control.component_type_registered(component_type)
         {
             let factory = self.control.spawn_control.store_factories.remove(&component_type).unwrap();
             let component_store = factory();
 
-            self.components.register_boxed_component_type(component_type, component_store);
+            self.components.control.register_boxed_component_type(component_type, component_store);
         }
     }
     
@@ -2063,15 +2224,19 @@ impl<T : SceneType + 'static> Scene<T>
             }
         }
         
-        if self.control.deferred_component_adds.len() > 0
+        if !self.control.spawn_control.deferred_adds.is_empty()
         {
-            let mut list = Vec::new();
-            std::mem::swap(&mut list, &mut self.control.deferred_component_adds);
-            
-            for (component_type, entity, component) in list.drain(..)
+            let mut deferred_component_adds = DeferredComponentAdds::new();
+            std::mem::swap(&mut deferred_component_adds, &mut self.control.spawn_control.deferred_adds);
+
+            for list in &mut deferred_component_adds.lists
             {
-                self.add_component_boxed(&entity, component, component_type);
+                list.add_to_scene(&mut self.components);
             }
+
+            std::mem::swap(&mut deferred_component_adds, &mut self.control.spawn_control.deferred_adds);
+
+            self.control.spawn_control.deferred_adds.clear();
         }
 
         if self.control.deferred_component_removals.len() > 0
@@ -2081,7 +2246,7 @@ impl<T : SceneType + 'static> Scene<T>
 
             for (component_type, entity) in &list
             {
-                self.remove_component_internal(entity, *component_type);
+                self.components.remove_component(entity, *component_type);
             }
         }
 
@@ -2096,14 +2261,14 @@ impl<T : SceneType + 'static> Scene<T>
             }
         }
 
-        if self.need_purge
+        if self.components.need_purge
         {
-            for index in &self.stores_to_purge
+            for index in &self.components.stores_to_purge
             {
-                self.components.component_stores[index].borrow_mut().purge_entities();
+                self.components.control.component_stores[index].borrow_mut().purge_entities();
             }
-            self.stores_to_purge.clear();
-            self.need_purge = false;
+            self.components.stores_to_purge.clear();
+            self.components.need_purge = false;
         }
         
         self.control.deferred_flag = false;
