@@ -2,7 +2,7 @@ use crate::gameloop::{GameInfo, GameControl, GameControlOptions};
 use crate::renderer::{DrawTransform, DrawBackgroundColorInfo};
 use crate::scene::{BaseScene, SimpleScene, NullSceneType, Entity, Component, ThinkerSystem, DrawerSystem, ThinkerArgs, DrawerArgs};
 use rand::seq::SliceRandom;
-use keeshond_datapack::DataMultistore;
+use keeshond_datapack::{DataId, DataMultistore};
 
 #[derive(Debug)]
 struct FooComponent { runs : u32 }
@@ -198,7 +198,7 @@ impl DrawerSystem for DrawSystemGood
                 g : 1.0,
                 b : 1.0,
                 alpha : 1.0,
-                shader_id : 0
+                shader_id : DataId::new(0)
             });
         }
     }
@@ -231,7 +231,7 @@ impl DrawerSystem for DrawSystemBad
                 g : 1.0,
                 b : 1.0,
                 alpha : 1.0,
-                shader_id : 0
+                shader_id : DataId::new(0)
             });
         }
     }
