@@ -6,6 +6,55 @@ use serde::{Deserialize, Deserializer, Serialize, Serializer};
 use std::ops::{Add, Sub, AddAssign, SubAssign, MulAssign};
 use std::fmt::Debug;
 
+const ANGLE_UP_RIGHT_RIGHT : f64 = -std::f64::consts::FRAC_PI_8;
+const ANGLE_UP_RIGHT : f64 = -std::f64::consts::FRAC_PI_4;
+const ANGLE_UP_UP_RIGHT : f64 = -std::f64::consts::FRAC_PI_4 - std::f64::consts::FRAC_PI_8;
+const ANGLE_UP_UP_LEFT : f64 = -std::f64::consts::FRAC_PI_2 - std::f64::consts::FRAC_PI_8;
+const ANGLE_UP_LEFT : f64 = -std::f64::consts::FRAC_PI_2 - std::f64::consts::FRAC_PI_4;
+const ANGLE_UP_LEFT_LEFT : f64 = -std::f64::consts::FRAC_PI_2 - std::f64::consts::FRAC_PI_4 - std::f64::consts::FRAC_PI_8;
+const ANGLE_DOWN_LEFT_LEFT : f64 = std::f64::consts::FRAC_PI_2 + std::f64::consts::FRAC_PI_4 + std::f64::consts::FRAC_PI_8;
+const ANGLE_DOWN_LEFT : f64 = std::f64::consts::FRAC_PI_2 + std::f64::consts::FRAC_PI_4;
+const ANGLE_DOWN_DOWN_LEFT : f64 = std::f64::consts::FRAC_PI_2 + std::f64::consts::FRAC_PI_8;
+const ANGLE_DOWN_DOWN_RIGHT : f64 = std::f64::consts::FRAC_PI_4 + std::f64::consts::FRAC_PI_8;
+const ANGLE_DOWN_RIGHT : f64 = std::f64::consts::FRAC_PI_4;
+const ANGLE_DOWN_RIGHT_RIGHT : f64 = std::f64::consts::FRAC_PI_8;
+
+#[derive(Debug, Clone, Copy, PartialEq, Eq)]
+pub enum InputDirection4
+{
+    Left,
+    Right,
+    Up,
+    Down
+}
+
+#[derive(Debug, Clone, Copy, PartialEq, Eq)]
+pub enum InputDirection8
+{
+    Left,
+    UpLeft,
+    Up,
+    UpRight,
+    Right,
+    DownRight,
+    Down,
+    DownLeft
+}
+
+impl From<InputDirection4> for InputDirection8
+{
+    fn from(dir_4 : InputDirection4) -> Self
+    {
+        match dir_4
+        {
+            InputDirection4::Left => { InputDirection8::Left }
+            InputDirection4::Right => { InputDirection8::Right }
+            InputDirection4::Up => { InputDirection8::Up }
+            InputDirection4::Down => { InputDirection8::Down }
+        }
+    }
+}
+
 #[derive(Debug, Clone, Copy, Default, Eq, PartialEq, Serialize, Deserialize)]
 pub struct Rect<T : Copy + Add<Output = T> + AddAssign + SubAssign + MulAssign + PartialOrd>
 {
@@ -129,6 +178,108 @@ impl<T : Debug + Copy + Sub<Output = T> + AddAssign + SubAssign + MulAssign + Pa
     }
 }
 
+pub fn xy_to_angle(x : f64, y : f64) -> Option<f64>
+{
+    if x == 0.0 && y == 0.0
+    {
+        return None;
+    }
+
+    Some(y.atan2(x))
+}
+
+pub fn xy_to_distance(x : f64, y : f64) -> f64
+{
+    (x.powi(2) + y.powi(2)).sqrt()
+}
+
+pub fn xy_to_distance_clamped(x : f64, y : f64) -> f64
+{
+    (x.powi(2) + y.powi(2)).sqrt().min(1.0)
+}
+
+pub fn angle_to_four_way_direction(angle : f64) -> Option<InputDirection4>
+{
+    if angle < ANGLE_UP_LEFT || angle >= ANGLE_DOWN_LEFT
+    {
+        return Some(InputDirection4::Left)
+    }
+    else if angle >= ANGLE_UP_RIGHT && angle < ANGLE_DOWN_RIGHT
+    {
+        return Some(InputDirection4::Right)
+    }
+    else if angle >= ANGLE_UP_LEFT && angle < ANGLE_UP_RIGHT
+    {
+        return Some(InputDirection4::Up)
+    }
+    else if angle >= ANGLE_DOWN_RIGHT && angle < ANGLE_DOWN_LEFT
+    {
+        return Some(InputDirection4::Down)
+    }
+
+    None
+}
+
+pub fn angle_to_four_way_direction_deadzone(angle : f64, deadzone : f64) -> Option<InputDirection4>
+{
+    if angle < ANGLE_UP_LEFT - deadzone || angle > ANGLE_DOWN_LEFT + deadzone
+    {
+        return Some(InputDirection4::Left)
+    }
+    else if angle > ANGLE_UP_RIGHT + deadzone && angle < ANGLE_DOWN_RIGHT - deadzone
+    {
+        return Some(InputDirection4::Right)
+    }
+    else if angle > ANGLE_UP_LEFT + deadzone && angle < ANGLE_UP_RIGHT - deadzone
+    {
+        return Some(InputDirection4::Up)
+    }
+    else if angle > ANGLE_DOWN_RIGHT + deadzone && angle < ANGLE_DOWN_LEFT - deadzone
+    {
+        return Some(InputDirection4::Down)
+    }
+
+    None
+}
+
+pub fn angle_to_eight_way_direction(angle : f64) -> Option<InputDirection8>
+{
+    if angle < ANGLE_UP_LEFT_LEFT || angle >= ANGLE_DOWN_LEFT_LEFT
+    {
+        return Some(InputDirection8::Left)
+    }
+    else if angle >= ANGLE_UP_LEFT_LEFT && angle < ANGLE_UP_UP_LEFT
+    {
+        return Some(InputDirection8::UpLeft)
+    }
+    else if angle >= ANGLE_UP_UP_LEFT && angle < ANGLE_UP_UP_RIGHT
+    {
+        return Some(InputDirection8::Up)
+    }
+    else if angle >= ANGLE_UP_UP_RIGHT && angle < ANGLE_UP_RIGHT_RIGHT
+    {
+        return Some(InputDirection8::UpRight)
+    }
+    else if angle >= ANGLE_UP_RIGHT_RIGHT && angle < ANGLE_DOWN_RIGHT_RIGHT
+    {
+        return Some(InputDirection8::Right)
+    }
+    else if angle >= ANGLE_DOWN_RIGHT_RIGHT && angle < ANGLE_DOWN_DOWN_RIGHT
+    {
+        return Some(InputDirection8::DownRight)
+    }
+    else if angle >= ANGLE_DOWN_DOWN_RIGHT && angle < ANGLE_DOWN_DOWN_LEFT
+    {
+        return Some(InputDirection8::Down)
+    }
+    else if angle >= ANGLE_DOWN_DOWN_LEFT && angle < ANGLE_DOWN_LEFT_LEFT
+    {
+        return Some(InputDirection8::DownLeft)
+    }
+
+    None
+}
+
 fn sort_map<S, T : Serialize + Ord, U : Serialize>(map : &HashMap<T, U>, serializer : S) -> Result<S::Ok, S::Error> where S : Serializer
 {
     let sorted : BTreeMap<_, _> = map.iter().collect();
